package Lingua::Thesaurus::IO::ISO2788;
use Moose;
with 'Lingua::Thesaurus::IO';

die "not implemented yet!";

1;

__END__

=head1 NAME

Lingua::Thesaurus::IO::ISO2788 - IO class for ISO thesauri

=head1 DESCRIPTION

This class will parse thesauri in ISO format.
Not implemented yet.



